<?php
/**
 * @version		$Id: edit_process.php,v 1.4 2012-03-16 15:33:14 juanca Exp $
 * @package		vpl. process submission edit
 * @copyright	Copyright (C) 2009 Juan Carlos Rodríguez-del-Pino. All rights reserved.
 * @license		GNU/GPL, see LICENSE.txt or http://www.gnu.org/licenses/gpl-2.0.html
 * @author		Juan Carlos Rodriguez-del-Pino
 **/

global $CFG;

require_once dirname(__FILE__).'/../../../config.php';
require_once $CFG->libdir.'/formslib.php';
require_once $CFG->libdir.'/moodlelib.php';
require_once dirname(__FILE__).'/../locallib.php';
require_once dirname(__FILE__).'/../vpl.class.php';
require_once dirname(__FILE__).'/../vpl_submission_CE.class.php';
require_once dirname(__FILE__).'/../vpl_example_CE.class.php';

if(!isloggedin()){
	print_header();
	vpl_js_alert(get_string('loggedinnot'));
	print_error('loggedinnot');
}
$id = required_param('id',PARAM_INT);
$vpl = new mod_vpl($id);
$userid = optional_param('userid',FALSE,PARAM_INT);
if(!$userid || $userid == $USER->id){//Edit own submission
	$userid = $USER->id;
	$vpl->require_capability(VPL_SUBMIT_CAPABILITY,true);
}
else { //Edit other user submission
	$vpl->require_capability(VPL_MANAGE_CAPABILITY,true);
}

//Display page
if(!$vpl->is_submit_able() || !$vpl->pass_password_check() || !$vpl->pass_network_check()){
	vpl_js_alert(get_string('nopermissions'));
	print_error('nopermissions');
}
$vpl->print_header(get_string('edit',VPL));
echo vpl_include_jsfile('transferResult.js');
echo "\n";
if($fontsize=optional_param('fontsize',FALSE)){
	set_user_preference('vpl_fontsize', $fontsize);
}
if(optional_param('save',FALSE)){
	if($userid != $USER->id){
		$vpl->require_capability(VPL_MANAGE_CAPABILITY);
	}
	mod_vpl_submission_CE::send_start_process(get_string('saving',VPL));
	$vpl->add_to_log('save submission', vpl_rel_url('forms/edit_process.php','id',$id,'userid',$userid));
	$raw_POST_size = strlen(file_get_contents("php://input")); 
	if($_SERVER['CONTENT_LENGTH'] != $raw_POST_size){
		$error="NOT SAVED (Http POST error: CONTENT_LENGTH expected ".$_SERVER['CONTENT_LENGTH']." found $raw_POST_size)";
		mod_vpl_submission_CE::send_end_process($error);
		vpl_js_alert($error);
		print_box($error);
	}
	$files = array();
	$max = $vpl->get_instance()->maxfiles;
	for( $i = 0; $i < $max; $i++){
		$field_filename = 'filename'.$i;
		$field_filedata = 'filedata'.$i;
		if(isset($_POST[$field_filename]) && isset($_POST[$field_filedata])){
			$filename =$_POST[$field_filename];
			if($filename == '') break;
			$files[] = array('name' => $filename,
						'data' =>urldecode(stripslashes($_POST[$field_filedata])));
		}
		mod_vpl_submission_CE::send_alive();
	}
	$error='';
	if(!$vpl->add_submission($userid,$files,'',$error)){
		mod_vpl_submission_CE::send_end_process($error);
		vpl_js_alert($error);
		print_box($error);
	}
	mod_vpl_submission_CE::send_end_process();
}
else if(optional_param('run',FALSE)){
	if($userid != $USER->id || !$vpl->get_instance()->run){
		$vpl->require_capability(VPL_GRADE_CAPABILITY);
	}
	$vpl->add_to_log('run', vpl_rel_url('forms/edit.php','id',$id,'userid',$userid));
	$subinstance = $vpl->last_user_submission($userid);
	if($subinstance !== false || $vpl->get_instance()->example){
		if($vpl->get_instance()->example){
			$submission=new mod_vpl_example_CE($vpl);
		}else{
			$submission=new mod_vpl_submission_CE($vpl,$subinstance);
		}
		$submission->send_start_process(get_string('running',VPL));
		try{
			$submission->run(0);
		}catch(Exception $e){
			vpl_js_alert($e->getMessage());
		}
		$submission->send_end_process();
	}else{
		vpl_js_alert(get_string('nosubmission',VPL));
		mod_vpl_submission_CE::send_end_process();
	}
}
else if(optional_param('debug',FALSE)){
	if($userid != $USER->id || !$vpl->get_instance()->debug){
		$vpl->require_capability(VPL_GRADE_CAPABILITY);
	}
	$vpl->add_to_log('debug', vpl_rel_url('forms/edit.php','id',$id,'userid',$userid));
	$subinstance = $vpl->last_user_submission($userid);
	if($subinstance !== false || $vpl->get_instance()->example){
		if($vpl->get_instance()->example){
			$submission=new mod_vpl_example_CE($vpl);
		}else{
			$submission=new mod_vpl_submission_CE($vpl,$subinstance);
		}
		$submission->send_start_process(get_string('debugging',VPL));
		try{
			$submission->run(1);
		}catch(Exception $e){
			vpl_js_alert($e->getMessage());
		}
		$submission->send_end_process();
	}else{
		vpl_js_alert(get_string('nosubmission',VPL));
		mod_vpl_submission_CE::send_end_process();
	}
}
else if(optional_param('evaluate',FALSE)){
	if($userid != $USER->id || !$vpl->get_instance()->evaluate){
		$vpl->require_capability(VPL_GRADE_CAPABILITY);
	}
	$vpl->add_to_log('evaluate', vpl_rel_url('forms/edit.php','id',$id,'userid',$userid));
	$subinstance = $vpl->last_user_submission($userid);
	if($subinstance !== false){
		$submission=new mod_vpl_submission_CE($vpl,$vpl->last_user_submission($userid));
		$submission->send_start_process(get_string('evaluating',VPL));
		try{
			$submission->evaluate();
		}catch(Exception $e){
			vpl_js_alert($e->getMessage());
		}
		$submission->send_end_process();
	}else{
		vpl_js_alert(get_string('nosubmission',VPL));
		mod_vpl_submission_CE::send_end_process();
	}
}
else{
	$vpl->add_to_log('error post', vpl_rel_url('forms/edit.php','id',$id,'userid',$userid));
	vpl_js_alert(get_string('error'));
	print_box('error');
}
$vpl->print_footer();
?>