<?php
/**
 * @version		$Id: index.php,v 1.31 2012-01-18 16:41:02 juanca Exp $
 * @package		vpl. List all instances in a course
 * @copyright	Copyright (C) 2009 Juan Carlos Rodríguez-del-Pino. All rights reserved.
 * @license		GNU/GPL, see LICENSE.txt or http://www.gnu.org/licenses/gpl-2.0.html
 * @author		Juan Carlos Rodriguez-del-Pino
 **/

require_once dirname(__FILE__).'/../../config.php';
require_once dirname(__FILE__).'/locallib.php';
require_once dirname(__FILE__).'/list_util.class.php';
require_once dirname(__FILE__).'/vpl_submission.class.php';
$id = required_param('id', PARAM_INT);   // course
require_login($id); //Check login state

$sort=vpl_get_set_session_var('sort','');
$sortdir = vpl_get_set_session_var('sortdir','down');
$instanceselection = vpl_get_set_session_var('selection','all');

//Check course existence
if (! $course = get_record("course", "id", $id)) {
	print_error('invalidcourseid','',$id);
}

add_to_log($course->id, VPL, 'view all', "index.php?id=$course->id", '');
//Load strings
$burl = vpl_rel_url(basename(__FILE__),'id',$id);
$strname  				= get_string('name').' '.vpl_list_util::vpl_list_arrow($burl,'name',$instanceselection,$sort,$sortdir);
$strvpls 				= get_string('modulenameplural',VPL);
$strshortdescription 	= get_string('shortdescription', VPL).' '.vpl_list_util::vpl_list_arrow($burl,'shortdescription',$instanceselection,$sort,$sortdir);
$strvisiblefrom			= get_string('visiblefrom', VPL).' '.vpl_list_util::vpl_list_arrow($burl,'visiblefrom',$instanceselection,$sort,$sortdir);
$stravailablefrom		= get_string('availablefrom', VPL).' '.vpl_list_util::vpl_list_arrow($burl,'availablefrom',$instanceselection,$sort,$sortdir);
$strduedate				= get_string('duedate', VPL).' '.vpl_list_util::vpl_list_arrow($burl,'duedate',$instanceselection,$sort,$sortdir);
$strnopls 				= get_string('novpls', VPL);

$navlinks = array();
$navlinks[] = array('name' => $strvpls, 'link' => '', 'type' => 'activity');
$navigation = build_navigation($navlinks);
$CFG->stylesheets[] = vpl_mod_href('css/index.css');
print_header($strvpls, $course->fullname, $navigation, '', '', false, '', navmenu($course));

//Print selection by instance state

echo '&nbsp;';
popup_form(vpl_mod_href('index.php','id',$id,'sort',$sort,'sortdir',$sortdir).'&amp;selection=',
array('all' => get_string('all'),
	'open' => get_string('open',VPL),'closed' => get_string('closed',VPL),
	'timelimited' => get_string('timelimited',VPL),'timeunlimited' => get_string('timeunlimited',VPL),
	'automaticgrading' => get_string('automaticgrading',VPL),'manualgrading' => get_string('manualgrading',VPL),
	'examples' => get_string('examples',VPL)),
'selectionmenu',
$instanceselection, '', '', '', false, 'self', get_string('instanceselection',VPL));
echo '<br />';
@ob_flush();
flush();

if (!$cms = get_coursemodules_in_course(VPL, $course->id,"m.shortdescription, m.visiblefrom, m.availablefrom, m.duedate")) {
	notice($strnopls, vpl_abs_href('/course/view.php','id',$course->id));
	die;
}
$ovpls = get_all_instances_in_course(VPL,$course);
$timenow = time();
$vpls = array();
//Get and select vpls to show
foreach ($ovpls as $ovpl) {
	$vpl = new mod_vpl(false,$ovpl->id);
	$instance = $vpl->get_instance();
	if ($vpl->is_visible()) {
		switch($instanceselection){
			case 'all':
				$vpls[]=$vpl;
				break;
			case 'open':
				$min = $instance->availablefrom;
				$max = $instance->duedate == 0 ? PHP_INT_MAX:$instance->duedate;
				if($timenow >= $min && $timenow <= $max){
					$vpls[]=$vpl;
				}
				break;
			case 'closed':
				$min = $instance->availablefrom;
				$max = $instance->duedate == 0 ? PHP_INT_MAX:$instance->duedate;
				if($timenow < $min || $timenow > $max){
					$vpls[]=$vpl;
				}
				break;
			case 'timelimited':
				if($instance->duedate > 0){
					$vpls[]=$vpl;
				}
				break;
			case 'timeunlimited':
				if($instance->duedate == 0){
					$vpls[]=$vpl;
				}
				break;
			case 'automaticgrading':
				if($instance->grade !=0 && $instance->automaticgrading > 0){
					$vpls[]=$vpl;
				}
				break;
			case 'manualgrading':
				if($instance->grade !=0 && $instance->automaticgrading == 0){
					$vpls[]=$vpl;
				}
				break;
			case 'examples':
				if($instance->example){
					$vpls[]=$vpl;
				}
				break;
		}
	}
}
//Is the user a grader?
$grader = false;
$student = false;
$visiblefrom =false;
$availablefrom = false;
$duedate = false;
$no_grade = true;
foreach ($vpls as $vpl) {
	if($vpl->has_capability(VPL_GRADE_CAPABILITY)){
		$grader = true;
	}elseif($vpl->has_capability(VPL_SUBMIT_CAPABILITY)){
		$student =true;
	}
	$instance = $vpl->get_instance();
	if($instance->grade!=0 && !$instance->example){
		$no_grade=false;
	}
	if($instance->visiblefrom>0
		&& $instance->visiblefrom != $instance->availablefrom){
		$visiblefrom=true;
	}
	if($instance->availablefrom>0){
		$availablefrom=true;
	}
	if($instance->duedate>0){
		$duedate=true;
	}
}
//if no instance with grade
$grader = $grader && !$no_grade;
$student = $student && !$no_grade;

//usort of old PHP versions don't call static class functions
if($sort>''){
	$corder = new vpl_list_util;
	$corder->set_order($sort,$sortdir=='down');
	usort($vpls,array($corder,'cpm'));
}
//Generate table
$table->head  = array ('#',$strname, $strshortdescription);
$table->align = array ('left','left', 'left');
if($visiblefrom){
	$table->head[]  = $strvisiblefrom;
	$table->align[] = 'center';
}
if($availablefrom){
	$table->head[]  = $stravailablefrom;
	$table->align[] = 'center';
}
if($duedate){
	$table->head[]  = $strduedate;
	$table->align[] = 'center';
}
if($grader && !$no_grade){
	$table->head[] = get_string('submissions',VPL);
	$table->head[] = get_string('graded',VPL);
	$table->align[] = 'right';
	$table->align[] = 'right';
}
if($student && !$no_grade){
	$table->head[] = get_string('grade');
	$table->align[] = 'left';
}
$table->data = array();
$totalsubs=0;
$totalgraded=0;
foreach ($vpls as $vpl) {
	$instance = $vpl->get_instance();
	$url = vpl_rel_url('view.php','id',$vpl->get_course_module()->id);
	$row = array (count($table->data)+1,"<a href='$url'>{$vpl->get_printable_name()}</a>",
			$instance->shortdescription);
	if($visiblefrom){
		$row[] = $instance->visiblefrom>0?userdate($instance->visiblefrom):'';
	}
	if($availablefrom){
		$row[] = $instance->availablefrom>0?userdate($instance->availablefrom):'';
	}
	if($duedate){
		$row[] = $instance->duedate>0?userdate($instance->duedate):'';
	}
	if($grader){
		if($vpl->has_capability(VPL_GRADE_CAPABILITY)
			&& $instance->grade != 0
			&& !$instance->example){
			$info = vpl_list_util::count_graded($vpl);
			$totalsubs += $info['submissions'];
			$totalgraded += $info['graded'];
			$row[]=$info['submissions'];
			//Need mark?
			if( $info['submissions'] > $info['graded'] &&
			    $instance->grade != 0 &&
			    $instance->automaticgrading == 0 &&
			    !($instance->duedate != 0 && $instance->duedate > time())){
			    $url = vpl_rel_url('views/submissionslist.php','id',$vpl->get_course_module()->id,'selection','notgraded');
			    $diff=$info['submissions'] - $info['graded'];
				$row[]='<div class="vpl_nm">'.$info['graded'].' <a href="'.$url.'">('.$diff.')</a><div>';
			}else{
				//No grade able
				if($instance->grade == 0 && $info['graded']==0){
					$row[]='-';
				}else{
					$row[]=$info['graded'];
				}
			}
		}else{
			$row[]='';
			$row[]='';
		}
	}
	if($student){
		if(!$vpl->has_capability(VPL_GRADE_CAPABILITY)
			&& $vpl->has_capability(VPL_SUBMIT_CAPABILITY)
			&& $instance->grade != 0
			&& !$instance->example){
			$subinstance = $vpl->last_user_submission($USER->id);
			if($subinstance){ //Submitted
				$submission = new mod_vpl_submission($vpl,$subinstance);
				if($subinstance->dategraded>0 && $instance->visiblegrade){
					$text = $submission->print_grade_core();
				}else{
					$result=$submission->getCE();
					$text='';
					if($result['executed']!==0){
						$prograde=$submission->proposedGrade($result['execution']);
						if($prograde>''){
							$text=get_string('proposedgrade',VPL,$submission->print_grade_core($prograde));
						}
					}else{
						$text=get_string('nograde');
					}
				}
			}else{//No submitted
				$text = get_string('nosubmission',VPL);
				if($vpl->is_submit_able()){
					$text = '<div class="vpl_nm">'.$text.'</div>';
				}
			}
			$row[]=$text;
		}else{
			$row[]='-';
		}
	}else{
		
	}
	$table->data[] = $row;
}
if($totalsubs>0){
	$row = array ('','','');
	if($visiblefrom){
		$row[] = '';
	}
	if($availablefrom){
		$row[] = '';
	}
	if($duedate){
		$row[] = '';
	}
	end($row);
	$row[key($row)] = get_string('total');
	$row[] = $totalsubs;
	$row[] = $totalgraded;
	$table->data[] = $row;
}
echo "<br />";
print_table($table);
print_footer($course);

?>